@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2012 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
  */
 
 /*
@@ -194,6 +194,9 @@ bool sieve_variable_get_modifiable
 bool sieve_variable_assign
 	(struct sieve_variable_storage *storage, unsigned int index,
 		const string_t *value);
+bool sieve_variable_assign_cstr
+	(struct sieve_variable_storage *storage, unsigned int index,
+		const char *value);
 bool sieve_variable_get_identifier
 	(struct sieve_variable_storage *storage, unsigned int index,
 		const char **identifier);
